###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
from config import (
    print_message,
    )


def get_project_url(package_name, lp):
    """ Return the link to the project with the supplied name. """

    project = lp.projects[package_name]
    return project.self_link


def file_bug(package_name, description, title, tags, lp):
    """ File a bug on the package specified by name.

    Returns the newly created bug or None if no bug was filed.
    """
    project_url = get_project_url(package_name, lp)
    return file_bug_by_url(project_url, description, title, tags, lp)


def file_bug_by_url(target_url, description, title, tags, lp):
    """ File a bug on the bug target specified by target_url.

    Returns the newly created bug or None if no bug was filed.
    """
    bug = lp.bugs.createBug(description=description,
                            title=title,
                            target=target_url,
                            tags=tags)
    if bug is not None:
        print 'Filed bug %i: %s' % (bug.id, bug.self_link)
    else:
        print "Error: unable to file bug '%s'." % title

    return bug


def bug_already_filed(package_name, lp):
    """ Check to see if this project already has an active bug."""
    project_url = get_project_url(package_name, lp)
    return bug_already_filed_by_url(project_url, lp)


def bug_already_filed_by_url(project_url, lp):
    """ Check to see if this project already has an active bug."""

    dupe_tags = ['ftbfs', 'arm-porting-queue']
    project = lp.load(project_url)

    # Should tags_combinator be 'Any'? Assumption now is to determine
    # duplicate if the bug is an ftbfs and it is in the arm-porting-queue.
    dupe_bugs = project.searchTasks(tags=dupe_tags, tags_combinator='All')

    return len(dupe_bugs[:1]) > 0
