###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
from testtools import TestCase

from testtools import TestCase
from launchpadlib.launchpad import Launchpad
from tests.fixtures import (
    TestCaseWithFixtures,
    CreateLaunchpadInstanceFixture,
    MockSomethingFixture,
    )
from data_parsing import (
    get_tags_for_fail,
    )
from bug_reporting import (
    bug_already_filed_by_url,
#    get_project_url,
#    file_bug,
    )
from config import (
    get_service_root,
    get_base_url,
    get_base_api_url,
    get_launchpad,
    set_launchpad,
    init,
    inited,
    ConfigInitError,
    LaunchpadError,
    )


class TestBugTags(TestCase):

    def setUp(self):
        super(TestBugTags, self).setUp()

    def test_default_tag_ftbfs(self):
        """ All bugs should get this tag. """
        tags = get_tags_for_fail("Dummy tag data")
        self.assertIn('ftbfs', tags)

    def test_default_tag_arm_porting_queue(self):
        """ All bugs should get this tag. """
        tags = get_tags_for_fail("Dummy tag data")
        self.assertIn('arm-porting-queue', tags)

    def test_no_match_no_extra_tag(self):
        """ This will not match, so should only get default tags. """
        tags = get_tags_for_fail("This will never trigger matcher.")
        self.assertIn('ftbfs', tags)
        self.assertIn('arm-porting-queue', tags)
        self.assertFalse(len(tags) > 2,
                        msg='Got more than default tags without match.')


#class TestLaunchpadProject(TestCaseWithFixtures):
#
#    def setUp(self):
#        super(TestLaunchpadProject, self).setUp()
#        fixture = self.useFixture(CreateLaunchpadInstanceFixture())
#        self.lp = fixture.get_lp_instance()
#
#    def test_verify_correct_project(self):
#        package_name_to_test = 'jetty'
#        url = get_project_url(package_name_to_test, self.lp)
#        project = self.lp.load(url)
#        package_name = project.name
#        self.assertEquals(package_name_to_test, package_name)


class TestFileBug(TestCaseWithFixtures):

    def setUp(self):
        super(TestFileBug, self).setUp()

    def test_duplicate_exists(self):
        class MockLaunchpad(object):

            def load(obj, url):
                class MockProject(object):
                    def searchTasks(obj, tags, tags_combinator):
                        return ['something here']
                return MockProject()

        self.assertTrue(bug_already_filed_by_url('https://dummy',
                                            MockLaunchpad()))

    def test_duplicate_does_not_exist(self):
        class MockLaunchpad(object):

            def load(obj, url):
                class MockProject(object):
                    def searchTasks(obj, tags, tags_combinator):
                        return []
                return MockProject()

        self.assertFalse(bug_already_filed_by_url('https://dummy',
                                            MockLaunchpad()))

#    def test_file_bug(self):
#        bug = file_bug(self.project_name, self.description, self.title,
#                       self.tags, self.lp)
#        self.assertTrue(bug is not None, msg='Failed to file bug')

#    def test_file_duplicate_bug(self):
#        bug = file_bug(self.project_name, self.description, self.title,
#                       self.tags, self.lp)
#        self.assertTrue(bug is None,
#        msg='Duplicate bug should not be filed')


class TestConfigInit(TestCaseWithFixtures):

    def setUp(self):
        super(TestConfigInit, self).setUp()
        inited = False

    def test_get_root_not_inited(self):
        self.assertRaises(ConfigInitError, get_service_root)

    def test_get_base_url_not_inited(self):
        self.assertRaises(ConfigInitError, get_base_url)

    def test_get_base_api_url_not_inited(self):
        self.assertRaises(ConfigInitError, get_base_api_url)

    def test_get_launchpad_not_inited(self):
        self.assertRaises(ConfigInitError, get_launchpad)

    def test_init_unsupported_lp(self):
        self.assertRaises(ConfigInitError, init, 'this_does_not_exist',
                          'test', '/dev/null')

    def test_set_launchpad_fail(self):
        def mock_func(obj, appid, root, cachedir):
            return None

        self.useFixture(MockSomethingFixture(
                Launchpad, 'login_with', classmethod(mock_func)))
        self.assertRaises(LaunchpadError, set_launchpad, 'staging',
                          'test', '/dev/null')
